# ip16adc.tcl---IP-16ADC IP pack diagnostic frame
# 

set debug 0
set started 0

set testmode 0
set monitor 0

set frame ip16adc
set frametitle "IP-16ADC Diagnostics Frame"
set icontitle "ip16adc"

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource vme}

set mname "IP16ADC"

set VIPC616BaseAddress 0xffffc000
set IPslot 0

#    8 adcs (differential setup)
#   16 adcs (single ended setup)
set adcs 8

set CSR 0

loadsource ipcommon


proc warn-wrongIP {} {
   midas-warning "This IP pack is not a 16ADC"
}

proc define-module {} {
  global resource mname IPslot VIPC616BaseAddress
  set-footer "deleting all IP-16ADC registers"
  EG "register :pattern '/$resource/$mname.*' !delete"

#     ID
      define-register xIP ManID        "-a $VIPC616BaseAddress -s $IPslot -o 0x09 -d id -p ro -w 8 -zx"
      define-register xIP ModID        "-a $VIPC616BaseAddress -s $IPslot -o 0x0b -d id -p ro -w 8 -zx"

      define-register xIP Control      "-a $VIPC616BaseAddress -s $IPslot -o 0x00 -d io -p wo -w 16 -zx"
      define-register xIP Status       "-a $VIPC616BaseAddress -s $IPslot -o 0x00 -d io -p ro -w 16 -zx"

      define-register xIP Data         "-a $VIPC616BaseAddress -s $IPslot -o 0x04 -d io -p ro -w 16 -zx"
      define-register xIP Convert      "-a $VIPC616BaseAddress -s $IPslot -o 0x0c -d io -p wo -w 16 -zx"

     click-redisplay
}

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc click-redisplay {} {

     set manid [read-register ManID]
     update-widget MANID $manid

     set modid [read-register ModID]
     update-widget MODID $modid

     if {$manid == 0xf0 && $modid == 0x36} then {set passed 1} else {set passed 0}
     update-widget PASSED $passed

     click-RCSR
}


proc reverse-bits b {
   set bits 0
   for {set i 0} {$i < 16} {incr i} {
      set bits [expr $bits | ((($b >> $i) & 1) << (15 - $i))]
   }
   set bits
}

proc click-CSR {v} {
    global CSR

    set CSR [reverse-bits $v]
}

proc click-RCSR {} {
    global CSR

    set CSR [expr ([read-register Status] & 0xfff0) + ($CSR & 0xf)]
    update-widget BITS [reverse-bits $CSR]
}

proc click-WCSR {} {
    global CSR
    write-register Control $CSR
}

proc read-adc {} {update-widget ADC [read-register Data]}

proc click-convert {} {

    write-register Convert -1
    read-adc
}

if {$adcs == 16} then {
#    single ended version
proc click-channel {v}  {
   global CSR
   set CSR [expr ($CSR & 0xffe0) + (((($v - 1) & 1) + 1) << 3)]
   set CSR [expr  $CSR           + (($v - 1)/2)]

    click-WCSR
    click-RCSR
}
} else {

#    differential  version
proc click-channel {v}  {
   global CSR
   set CSR [expr ($CSR & 0xffe0) + 0x18 + ($v - 1)]

    click-WCSR
    click-RCSR
}
}

proc click-range {v}  {
    global CSR
    set CSR [expr ($CSR & 0xff9f) + ($v << 5)]

    click-WCSR
    click-RCSR
}

proc click-reset {} {
    click-channel 1
    click-range 3
    update-widget CHANNEL 1
    update-widget RANGE 3
}

proc click arg {
   global debug
  global VIPC616BaseAddress IPslot

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      IPBASEADDR   {
          set VIPC616BaseAddress $v
          define-module
      }
      IPSLOTADDR {
          set IPslot $v
          define-module
      }

      BITS     {click-CSR $v}
      CHANNEL  {click-channel $v}
      RANGE    {click-range $v}

      default   {puts stdout $arg}
   }
}



proc make-frame {} {
  global env VIPC616BaseAddress IPslot adcs

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 25

      :text IPBASEADDR   :label "IP Carrier Base Address:" :value $VIPC616BaseAddress :w 10
      :choice IPSLOTADDR :label "IP Pack slot" :strings [list A B C D E F]         :value $IPslot
      :next-row T; :col-gap 20
      :text MANID        :label "Man ID"  :w 4
      :text MODID        :label "Mod ID"  :w 4
      :checkbox PASSED       :label "IP check passed"

if {$adcs == 16} then {
      :message M1 :label "single ended configuration - 16 adc channels"
} else {
      :message M1 :label "differential configuration - 8 adc channels"
}

   :panel CLICK :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 20

     :number CHANNEL :label "Adc Channel" :w 2 :min 1 :max $adcs
     :choice-stack RANGE :label "Adc Range" :strings [list "0V -> 10V" "0V -> 5V" "-10V -> +10V" "-5V -> +5V"]

       :next-row T
       :col-gap 5

   :choice BITS :label "Control & Status" :strings [list 15 14 13 12 11 10 9 8 7 6 5 4 3 2 1 0] :w 2 :exclusive F

          :button W    :label Write :w 5  :command click-WCSR
          :button R    :label Read  :w 5  :command click-RCSR

       :next-row T

      :text ADC :label "ADC data" :w 6
      :button RADC :label "Read" :command "read-adc"
   
   :panel TEST  :layout T :padx 10 :pady 10
      :layout H; :col-gap 15
      :button REDISPLAY :label "Redisplay"              :command "click-redisplay"
      :button CONVERT     :label "Force Convert"                  :command click-convert
      :button DEFINE    :label "Define IP-16ADC registers" :command "define-module"
      :button RESET     :label "Reset module"  :command click-reset
   
   :show T

}


proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
#      define-module
#      click-reset
   } else {open-frame}
}

proc Start {s} {
    global IPslot started

    set IPslot $s
    set started 0
    start
}


